unit RegExp;

interface

type
  HRegExp = type Pointer;

  TRegMatch = record
    Start: Cardinal;        { Start of match }
    Stop:  Cardinal;        { End of match }
  end;

const
  RE_OK                   = 0;
  RE_NOTFOUND             = 1;
  RE_INVALIDPARAMETER     = 2;
  RE_EXPRESSIONTOOBIG     = 3;
  RE_OUTOFMEMORY          = 4;
  RE_TOOMANYSUBEXPS       = 5;
  RE_UNMATCHEDPARENS      = 6;
  RE_INVALIDREPEAT        = 7;
  RE_NESTEDREPEAT         = 8;
  RE_INVALIDRANGE         = 9;
  RE_UNMATCHEDBRACKET     = 10;
  RE_TRAILINGBACKSLASH    = 11;
  RE_INTERNAL             = 20;
  RE_NOPROG               = 30;
  RE_NOSTRING             = 31;
  RE_NOMAGIC              = 32;
  RE_NOMATCH              = 33;
  RE_NOEND                = 34;
  RE_INVALIDHANDLE        = 99;

function RegComp(Pattern: string; var Error: Integer): HRegExp; stdcall;
function RegExec(RegEx: HRegExp; Str: string; var Match: TRegMatch): Integer; stdcall;
function RegError(Regex: HRegExp; Error: Integer; var ErrorBuf: string): Integer; stdcall;
procedure RegFree(Regex: HRegExp); stdcall;
function RegPointer(Regex: HRegExp): Pointer; stdcall;

implementation

uses
  RegExpObj;

resourcestring
  S_OK                = 'No error';
  S_NOTFOUND          = 'Not found';
  S_INVALIDPARAMETER  = 'Invalid parameter';
  S_EXPRESSIONTOOBIG  = 'Expression too big';
  S_OUTOFMEMORY       = 'Not enough memory';
  S_TOOMANYSUBEXPS    = 'Too many subexpresssions';
  S_UNMATCHEDPARENS   = 'Unmatched parentheses';
  S_INVALIDREPEAT     = 'Invalid repeat';
  S_NESTEDREPEAT      = 'Nested repeat';
  S_INVALIDRANGE      = 'Invalid range';
  S_UNMATCHEDBRACKET  = 'Unmatched bracket';
  S_TRAILINGBACKSLASH = 'Trailing backslash';
  S_INTERNAL          = 'Internal error - should not happen';
  S_NOPROG            = 'No compiled expression';
  S_NOSTRING          = 'No string';
  S_NOMAGIC           = 'Invalid compiled expression';
  S_NOMATCH           = 'No match';
  S_NOEND             = 'No end';
  S_INVALIDHANDLE     = 'Invalid handle';

function VerifyHandle(const Reg: HRegExp): Boolean;
begin
  Result := (Reg <> nil) and
            (PRegExp(Reg).Prog[0] = Chr(MAGIC));
end;

function RegComp(Pattern: string; var Error: Integer): HRegExp; stdcall;
begin
  Result := _regcomp(PChar(Pattern));
  if Result = nil then
    Error := _reggeterror
  else
    Error := RE_OK;
end;

function RegExec(RegEx: HRegExp; Str: string; var Match: TRegMatch): Integer; stdcall;
var
  PReg: PRegExp absolute RegEx;
begin
  _regseterror(0);
  if _regexec(PReg, PChar(str)) then
  begin
    Match.Start := Preg.StartP[0] - PChar(Str);
    Match.Stop := PReg.Endp[0] - PChar(Str);
    Result := RE_OK;
  end
  else
    if _reggeterror <> 0 then
      Result := _reggeterror
    else
      Result := RE_NOTFOUND;
end;

function RegError(Regex: HRegExp; Error: Integer; var ErrorBuf: string): Integer; stdcall;
begin
  if VerifyHandle(Regex) then
  begin
    case Error of
      RE_OK:                ErrorBuf := S_OK;
      RE_NOTFOUND:          ErrorBuf := S_NOTFOUND;
      RE_INVALIDPARAMETER:  ErrorBuf := S_INVALIDPARAMETER;
      RE_EXPRESSIONTOOBIG:  ErrorBuf := S_EXPRESSIONTOOBIG;
      RE_OUTOFMEMORY:       ErrorBuf := S_OUTOFMEMORY;
      RE_TOOMANYSUBEXPS:    ErrorBuf := S_TOOMANYSUBEXPS;
      RE_UNMATCHEDPARENS:   ErrorBuf := S_UNMATCHEDPARENS;
      RE_INVALIDREPEAT:     ErrorBuf := S_INVALIDREPEAT;
      RE_NESTEDREPEAT:      ErrorBuf := S_NESTEDREPEAT;
      RE_INVALIDRANGE:      ErrorBuf := S_INVALIDRANGE;
      RE_UNMATCHEDBRACKET:  ErrorBuf := S_UNMATCHEDBRACKET;
      RE_TRAILINGBACKSLASH: ErrorBuf := S_TRAILINGBACKSLASH;
      RE_INTERNAL:          ErrorBuf := S_INTERNAL;
      RE_NOPROG:            ErrorBuf := S_NOPROG;
      RE_NOSTRING:          ErrorBuf := S_NOSTRING;
      RE_NOMAGIC:           ErrorBuf := S_NOMAGIC;
      RE_NOMATCH:           ErrorBuf := S_NOMATCH;
      RE_NOEND:             ErrorBuf := S_NOEND;
      RE_INVALIDHANDLE:     ErrorBuf := S_INVALIDHANDLE;
    end;
    Result := RE_OK;
  end
  else
    Result := RE_INVALIDHANDLE;
end;

procedure RegFree(Regex: HRegExp); stdcall;
begin
  FreeMem(Pointer(Regex));
end;

function RegPointer(Regex: HRegExp): Pointer; stdcall;
begin
  Result := @PRegExp(Regex)^.Prog;
end;

end.











