unit RegExpObj;

interface

const
  NSUBEXP = 10;

  // The first byte of the regexp internal "program" is actually this magic
  // number; the start node begins in the second byte.
  MAGIC = 156;

// since we provide the code for _malloc, we can use FreeMem to free the
// PRegExp we get. But normally, a _regfree() would be nice.

type
  PRegExp = ^_RegExp;
  _RegExp = packed record
    StartP: array[0..NSUBEXP - 1] of PChar;
    EndP: array[0..NSUBEXP - 1] of PChar;
    RegStart: Char;             // Internal use only.
    RegAnch: Char;              // Internal use only.
    RegMust: PChar;             // Internal use only.
    RegMLen: Integer;           // Internal use only.
    Prog: array[0..0] of Char;  // Internal use only.
  end;

function _regcomp(exp: PChar): PRegExp; cdecl;
function _regexec(prog: PRegExp; str: PChar): LongBool; cdecl;
function _reggeterror: Integer; cdecl;
procedure _regseterror(Err: Integer); cdecl;

implementation

uses
  SysUtils;

{$LINK '.\C\regexp.obj'}

function _regcomp(exp: PChar): PRegExp; cdecl; external;
function _regexec(prog: PRegExp; str: PChar): LongBool; cdecl; external;
function _reggeterror: Integer; cdecl; external;
procedure _regseterror(Err: Integer); cdecl; external;

function _malloc(Size: Cardinal): Pointer; cdecl;
begin
  GetMem(Result, Size);
end;

function _strlen(const Str: PChar): Cardinal; cdecl;
begin
  Result := StrLen(Str);
end;

function _strcspn(s1, s2: PChar): Cardinal; cdecl;
label Bye;
var
  SrchS2: PChar;
  Len: Integer;
begin
  Len := 0;
  while S1^ <> #0 do
  begin
   SrchS2 := S2;
   while SrchS2^ <> #0 do
   begin
     if S1^ = SrchS2^ then
       goto Bye;
     Inc(SrchS2);
   end;
   Inc(S1);
   Inc(Len);
  end;
Bye:
  Result := Len;
end;

function _strchr(const S: PChar; C: Integer): PChar; cdecl;
begin
  Result := StrScan(S, Chr(C));
end;

function _strncmp(S1, S2: PChar; MaxLen: Cardinal): Integer; cdecl;
begin
  Result := StrLComp(S1, S2, MaxLen);
end;

end.


